{$A+,B-,D-,E-,G+,F-,I-,L-,N-,O-,P-,Q-,R-,S-,T-,V-,X-,Y-}
{$M 2048, 0, 131072}
uses dos;

type
  str7=string[7];

const
  igc:set of char = [#0, #9, #10, #12, #13, #32];
  dcs:set of char = [#0, #9, #10, #12, #13, #32];
  chars:set of char = [#48..#57, #64..#90, #97..#122, #128..#254];
  bufsize = 16384;
  amsg:string[57] = 'Text Files Comparer v1.11 V-1996 by V.Lidovski';
  cps:longint = 64; (* default compare portion size *)
  pocs:longint = 1000000;
  labl:longint = 4096; (* Look_Ahead_Buffer_Limit *)
  end_cmp_code:byte = 0;
  rm:string[6] = '_}';
  lm:string[6] = '{_';
  displi:boolean = true;
  hyphens:boolean = true;

var
  f:array[1..2]of file;
  fn:array[1..2]of string[64];
  frs:array[1..2]of longint; (* file Remainder Size *)
  bu:array[1..2]of array[0..bufsize-1]of char; (* Read Buffer *)
  inx, ccp, ocp, bs, cpo, hc, cbase, lip, lis, lin:array[1..2]of longint;
    (* Index, Current/Old Char Pos, Buffer Size, Char Pos Ofs *)
  libu:array[1..2]of array[1..400]of longint;
  libufsize:integer;
  pp:array[1..2]of longint;
  i, i1:integer;
  s:string[255];

procedure error(n:byte);
  begin
    case n of
      1:
      begin
        writeln('USAGE:');
        writeln('  tfc [<options>] <text1> <text2>');
        writeln('OPTIONS:');
        writeln('  /nm            -- don''t show markers');
        writeln('  /nn            -- don''t show line numbers');
        writeln('  /nc            -- don''t show context');
        writeln('  /nh            -- treat hyphens as ordinary signs');
        writeln('  /ec            -- exact compare');
        writeln('  /cd:<number>   -- compare depth (default=64)');
        writeln('  /sa:<number>   -- synchronization area (default=4096)');
        writeln('  /ic:<filename> -- file with ignored characters');
        writeln('  /rm:<string>   -- right marker (default="_}")');
        writeln('  /lm:<string>   -- left marker (default="{_")');
        writeln('  /?             -- show this information');
        writeln('EXAMPLES:');
        writeln(' tfc /nm text1.txt text2.txt /sa:1024');
        writeln(' tfc text1.txt /b text2.txt /cd:32 >res.txt');
        writeln(' tfc /rm:__ /lm:__ text1.txt text2.txt');
        writeln(' tfc /sa:8192 /cd:48 text1.txt text2.txt')
      end;
      2: writeln('ERROR: compare depth must have numeric value');
      3: writeln('ERROR: synchronization area must have numeric value');
      4: writeln('ERROR: ', fn[i], ' not found');
      5: writeln('ERROR: ', s, ' not found');
      6: writeln('ERROR: ignored characters file has incorrect format');
      7: writeln('ERROR: internal #1');
      8: writeln('ERROR: some parameters are too great')
    end;
    halt(n)
  end;

function min(a, b:longint):longint;
  begin
    if a<b then
      min := a
    else
      min := b
  end;

function max(a, b:longint):longint;
  begin
    if a > b then
      max := a
    else
      max := b
  end;

procedure init;
  var j, j1:integer;
  begin
    writeln(amsg);
    if paramcount < 2 then
      error(1);
    fn[1] := '';
    fn[2] := '';
    for i := 1 to paramcount do begin
      s := paramstr(i);
      for j := 1 to length(s) do
        s[j] := upcase(s[j]);
      if pos('/NM', s) > 0 then begin
        rm := '';
        lm := '';
        continue
      end;
      if pos('/EC', s) > 0 then begin
        igc := [];
        continue
      end;
      if pos('/NC', s) > 0 then begin
        pocs := 0;
        continue
      end;
      if pos('/NH', s) > 0 then begin
        hyphens := false;
        continue
      end;
      if pos('/NN', s) > 0 then begin
        displi := false;
        continue
      end;
      if pos('/CD:', s) > 0 then begin
        delete(s, 1, 4);
        val(s, cps, i1);
        if (i1 <> 0) or (cps < 0) then
          error(2);
        continue
      end;
      if pos('/SA:', s) > 0 then begin
        delete(s, 1, 4);
        val(s, labl, i1);
        if i1 <> 0 then
          error(3);
        continue
      end;
      if pos('/IC:', s)>0 then begin
        delete(s, 1, 4);
        assign(f[1], s);
        for i1 := 1 to length(s) do
          s[i1] := upcase(s[i1]);
{$I-}
        reset(f[1], 1);
        if ioresult<>0 then
{$I+}
          error(5);
        blockread(f[1], bu[1], filesize(f[1]));
        igc := [];
        i1 := 0;
        while i1 < filesize(f[1]) do begin
          s := '';
          while (bu[1][i1] <> #10) and (i1 < filesize(f[1])) do begin
            if bu[1][i1] in [#48..#57] then s := s + bu[1][i1];
            inc(i1)
          end;
          val(s, j, j1);
          if (j > 255)or(j < 0) then
            error(6);
          igc := igc + [chr(j)];
          inc(i1)
        end;
        close(f[1]);
        continue
      end;
      if pos('/RM:', s) > 0 then begin
        delete(s, 1, 4);
        rm := s;
        continue
      end;
      if pos('/LM:', s) > 0 then begin
        delete(s, 1, 4);
        lm := s;
        continue
      end;
      if pos('/?', s) > 0 then
        error(1);
      if fn[1] = '' then
        fn[1] := s
      else
        if fn[2] = '' then
          fn[2] := s
        else
          error(1)
    end;
    if fn[2] = '' then
      error(1);
    if labl + cps >= bufsize then
      error(8);
    libufsize := min((labl + cps) div 15, 400);
    for i := 1 to 2 do begin
      for i1 := 1 to length(fn[i]) do
        fn[i][i1] := upcase(fn[i][i1]);
      assign(f[i], fn[i]);
      if amsg[40] <> 'i' then
        exit;
{$I-}
      reset(f[i], 1);
      if ioresult <> 0 then
{$I+}
        error(4);
      frs[i] := filesize(f[i]);
      bs[i] := 0;
      ccp[i] := 0;
      cpo[i] := 0;
      pp[i] := 0;
      cbase[i] := 0;
      lis[i] := 2;
      lip[i] := 1;
      lin[i] := 1;
      for i1 := 1 to libufsize do
        libu[i][i1] := -1
    end
  end;

procedure finish;
  begin
    close(f[2]);
    close(f[1]);
    writeln;
    case end_cmp_code of
      0: writeln('***** Files are identical');
      1: writeln('***** ', fn[2], ' exceeds ', fn[1]);
      2: writeln('***** ', fn[1], ' exceeds ', fn[2]);
      3: writeln('***** Files are too differrent')
    end
  end;

function ntstpos(n:byte):boolean;
  var x:longint;
  begin
    if ccp[n] + inx[n] + cpo[n] >= bs[n] then begin
      x := ocp[n] - cps;
      if x > 0 then begin
        ocp[n] := cps;
        move(bu[n][x], bu[n][0], ccp[n] + inx[n] + cpo[n] - x);
        ccp[n] := ccp[n] - x;
        pp[n] := pp[n] - x;
        cbase[n] := cbase[n] + x;
        bs[n] := bs[n] - x
      end;
      x := bufsize - bs[n];
      if x > frs[n] then begin
        x := frs[n];
        frs[n] := 0
      end else
        frs[n] := frs[n] - x;
      if x = 0 then begin
        ntstpos := true;
        exit
      end;
      blockread(f[n], bu[n][bs[n]], x);
      bs[n] := bs[n] + x
    end;
    ntstpos := false
  end;

function tstnextc(n:byte):boolean;
  label l1;
  begin
    if ntstpos(n) then begin
 l1:  tstnextc := false;
      exit
    end;
    if bu[n][ccp[n] + inx[n] + cpo[n]] = #$d then
      tstnextc := true
    else
      goto l1;
    inc(ccp[n])
  end;

function gnc(n:byte):char; (* #9 -- EOF, #10 -- ७ *)
  label l1;
  var
    c:char;
    x:longint;
  begin
l1: if ntstpos(n) then begin
       gnc := #9;
       exit
    end;
    x := ccp[n] + inx[n] + cpo[n];
    c := bu[n][x];
    if (c = #10) and (cbase[n] + x > libu[n][lip[n]]) then begin
      if lis[n] < libufsize then
        inc(lis[n])
      else
        lis[n] := 1;
      if lip[n] < libufsize then
        inc(lip[n])
      else
        lip[n] := 1;
      libu[n][lip[n]] := cbase[n] + x;
      inc(lin[n])
    end;
    inc(ccp[n]);
    if hyphens and (c = '-')
      and (bu[n][ccp[n] + cpo[n] + inx[n] - 2] in chars)
      and tstnextc(n) then begin
      gnc := #10;
      exit
    end;
    if c in igc then goto l1;
    gnc := c
  end;

function scancmp1:byte;  (* 0-ok, 1-eof1, 2-eof2, 3-diff *)
  label l1, l2;
  var
    c1, c2:char;
    c:byte;
  begin
l1: ocp[1] := ccp[1];
    ocp[2] := ccp[2];
    c1 := gnc(1);
    c2 := gnc(2);
    case c1 of
      #9:
      case c2 of
        #9: c := 0;
      else
        c := 1
      end;
      #10:
      case c2 of
        #9: c := 2;
        #10, '-': goto l1
      else
        ccp[2] := ocp[2];
        goto l1
      end;
      '-':
      case c2 of
        #9: c := 2;
        #10, '-': goto l1
      else
        goto l2
      end;
    else
      case c2 of
        #9: c := 2;
        #10: begin
          ccp[1] := ocp[1];
          goto l1
        end;
        '-': goto l2
      else
        if c1 <> c2 then begin
l2:       c := 3;
          dec(ccp[1]);
          dec(ccp[2])
        end else
          goto l1
      end
    end;
    if c in [1, 2] then begin
      ccp[1] := ocp[1];
      ccp[2] := ocp[2]
    end;
    scancmp1 := c
  end;

function scancmp2:byte;  (* 0-ok, 1-eof1, 2-eof2, 3-diff *)
  label l1, l2, l3;
  var
    c1, c2:char;
    c:byte;
  begin
    hc[1] := 0;
    hc[2] := 0;
l1: c1 := gnc(1);
    c2 := gnc(2);
    case c1 of
      #9:
      case c2 of
        #9: c := 0;
      else
        c := 1
      end;
      #10:
      case c2 of
        #9: c := 2;
        #10, '-': goto l2
      else
        ccp[2] := ocp[2];
        inc(hc[1]);
        goto l1
      end;
      '-':
      case c2 of
        #9: c := 2;
        #10, '-': goto l2
      else
        goto l3
      end;
    else
      case c2 of
        #9: c := 2;
        #10: begin
          ccp[1] := ocp[1];
          inc(hc[2]);
          goto l1
        end;
        '-': goto l3
      else
        if c1 <> c2 then begin
l3:       c := 3;
          dec(ccp[1]);
          dec(ccp[2])
        end else
l2:       c := 4
      end
    end;
    if c in [1, 2] then begin
      ccp[1] := ocp[1];
      ccp[2] := ocp[2]
    end;
    scancmp2 := c
  end;

function prepstr(i:longint):str7;
  var s:str7;
  begin
    str(i, s);
    while length(s) < 6 do
      s := '0' + s;
    prepstr := s + ': ';
  end;

function nextli(i:longint):longint;
  begin
    if i < libufsize then
      nextli := i + 1
    else
      nextli := 1
  end;

function findline(n:byte;p:longint):str7;
  label 1;
  var
    i, i1:longint;
  begin
    if not displi then begin
      findline := '';
      exit
    end;
    i := lis[n];
    repeat
      i1 := i;
      i := nextli(i);
      if (libu[n][i1] < p)
        and ((libu[n][i] < libu[n][i1]) or (libu[n][i] >= p)) then begin
        if i1 > lip[n] then
          i := lin[n] - libufsize + 1 + i1 - lis[n]
        else
          i := lin[n] - lip[n] + i1;
        goto 1
      end
    until i=lis[n];
    error(7);
1:  findline := prepstr(i)
  end;

procedure outdiff(n:byte;lim, ep:longint);
  var
    i, i1:longint;
  begin
    writeln;
    writeln('***** ', fn[n]);
    i1 := -1;
    for i := ccp[n] - 1 downto pp[n] do
      if not(bu[n][i] in dcs) then begin
        i1 := i;
        break
      end;
    if i1 < 0 then
      s := lm
    else
      s := '';
    for i := pp[n] to ccp[n] - 1 do begin
      s := s + bu[n][i];
      if i = i1 then s := s + lm;
      if bu[n][i] = #10 then begin
        write(findline(n, cbase[n] + i), s);
        s := ''
      end
    end;
    if ep > 0 then begin
      for i := ccp[n] to ep - 1 do begin
        s := s + bu[n][i];
        if bu[n][i] = #10 then begin
          write(findline(n, cbase[n] + i), s);
          s := ''
        end
      end;
      while (bu[n][ep] in dcs) and (ep < lim) do begin
        s := s + bu[n][ep];
        if bu[n][ep] = #10 then begin
          write(findline(n, cbase[n] + ep), s);
          s := ''
        end;
        inc(ep)
      end;
      s := s + rm;
      for i := lim - 1 downto ep do
        if (bu[n][i] in dcs) or (i = ep) then begin
          lim := i + 1;
          break
        end;
      for i := ep to lim - 1 do begin
        s := s + bu[n][i];
        if bu[n][i] = #10 then begin
          write(findline(n, cbase[n] + i), s);
          s := ''
        end
      end
    end else
      for i := ccp[n] to lim - 1 do begin
        s := s + bu[n][i];
        if bu[n][i] = #10 then begin
          write(findline(n, cbase[n] + i), s);
          s := ''
        end
      end;
    if (s <> '') or ((rm = '') and (lm = '')) then
      write(findline(n, cbase[n]+lim-1), s)
  end;

function synchro:boolean;
  label
    l1, l2;
  var
    i:integer;
    sum:array[1..2]of longint;
  begin
    ocp[1] := ccp[1];
    ocp[2] := ccp[2];
    end_cmp_code := 4;
    for i := 1 to 2 do begin
      pp[i] := max(ccp[i] - cps, 0);
      if pocs = 0 then
        pp[i] := ccp[i];
      if pp[i] = 0 then
        if i = 1 then
          continue
        else
          break;
      while not (bu[i][pp[i]] in dcs) do
        if pp[i] < ccp[i] then
          inc(pp[i])
        else
          break
    end;
    repeat
      if (ccp[1] = 0) or (ccp[2] = 0) then
        break;
      for i := 1 to 2 do begin
        sum[i] := 0;
        for i1 := ccp[i] - 1 downto pp[i] do
          if not(bu[i][i1] in dcs) then
            inc(sum[i])
      end;
      i := 0;
      if sum[1] > sum[2] then
        i := 2
      else if sum[1] < sum[2] then
        i := 1;
      if i = 0 then
        goto l1;
      while bu[i][pp[i]] in dcs do
        if pp[i] > 0 then
          dec(pp[i])
        else
          break;
      while not(bu[i][pp[i]] in dcs) do
        if pp[i] > 0 then
          dec(pp[i])
        else
          break
    until false;
l1: if inx[1] < min(labl, bs[1] + frs[1] - ccp[1] - cps) then (* $$ *)
      inc(inx[1])
    else if inx[2] < min(labl, bs[2] + frs[2] - ccp[2] - cps) then begin
      inc(inx[2]);
      inx[1] := 0
    end else if (inx[1] = labl) and (inx[2] = labl) then begin
      outdiff(1, ccp[1] + cps, 0);
      writeln('<...>');
      outdiff(2, ccp[2] + cps, 0);
      writeln('<...>');
      synchro := false;
      end_cmp_code := 3;
      exit
    end else begin
      outdiff(1, bs[1], 0);
      writeln('_}<EOF>');
      outdiff(2, bs[2], 0);
      writeln('_}<EOF>');
      synchro := false;
      exit
    end;
    cpo[1] := 0;
    cpo[2] := 0;
l2: case scancmp2 of
      0, 4: if cpo[1]<cps then begin
        cpo[1] := cpo[1] + ccp[1] - ocp[1] + hc[1];
        cpo[2] := cpo[2] + ccp[2] - ocp[2] + hc[2];
        ccp[1] := ocp[1];
        ccp[2] := ocp[2];
        goto l2
      end else begin
        ccp[1] := ocp[1];
        ccp[2] := ocp[2];
        outdiff(1, ccp[1] + inx[1] + min(cpo[1], pocs), ccp[1] + inx[1]);
        writeln;
        outdiff(2, ccp[2]+inx[2]+min(cpo[2], pocs), ccp[2]+inx[2]);
        writeln;
        for i := 1 to 2 do begin
          ccp[i] := ccp[i] + inx[i] + cpo[i];
          inx[i] := 0;
          cpo[i] := 0
        end;
        synchro := true
      end;
      1: begin
        end_cmp_code := 1;
        outdiff(1, ccp[1] + min(cpo[1], pocs), 0);
        writeln('_}<EOF>');
        outdiff(2, ccp[2] + min(cpo[2], pocs), 0);
        writeln('_} <...>');
        synchro := false
      end;
      2: begin
        end_cmp_code := 2;
        outdiff(1, ccp[1] + min(cpo[1], pocs), 0);
        writeln('_} <...>');
        outdiff(2, ccp[2] + min(cpo[2], pocs), 0);
        writeln('_}<EOF>');
        synchro := false
      end;
      3: begin
        ccp[1] := ocp[1];
        ccp[2] := ocp[2];
        goto l1
      end
    end
  end;

procedure main;
  begin
    repeat
      case scancmp1 of
        0: exit;
        1: begin
          end_cmp_code := 1;
          exit
        end;
        2: begin
          end_cmp_code := 2;
          exit
        end;
        3: if not synchro then
          exit;
      end
    until false
  end;

begin
  init;
  main;
  finish
end.
